@@ -59,8 +59,7 @@ import javax.servlet.http.HttpSession;
 import javax.servlet.http.HttpSessionBindingEvent;
 import javax.servlet.http.HttpSessionBindingListener;
 import javax.servlet.http.HttpSessionContext;
-import com.openexchange.ajp13.AJPv13ServiceRegistry;
-import com.openexchange.ajp13.servlet.http.HttpSessionWrapper;
+import com.openexchange.ajp13.Services;
 import com.openexchange.config.ConfigTools;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.java.HashKeyMap;
@@ -87,11 +86,11 @@ public class HttpSessionWrapper implements HttpSession {
             synchronized (HttpSessionWrapper.class) {
                 tmp = cookieTTL;
                 if (tmp < 0) {
-                    final ConfigurationService service = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
+                    final ConfigurationService service = Services.getService(ConfigurationService.class);
                     if (null == service) {
-                        tmp = (int) ConfigTools.parseTimespan("1W");
+                        tmp = ConfigTools.parseTimespanSecs("1W");
                     } else {
-                        tmp = (int) (ConfigTools.parseTimespan(service.getProperty("com.openexchange.cookie.ttl", "1W")) / 1000);
+                        tmp = ConfigTools.parseTimespanSecs(service.getProperty("com.openexchange.cookie.ttl", "1W"));
                     }
                     cookieTTL = tmp;
                 }
@@ -143,7 +142,7 @@ public class HttpSessionWrapper implements HttpSession {
         /*
          * Max. inactive interval
          */
-        final ConfigurationService service = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
+        final ConfigurationService service = Services.getService(ConfigurationService.class);
         maxInactiveIntervall = null == service ? 1800 : service.getIntProperty("com.openexchange.servlet.maxInactiveIntervall", 1800); // 30 Minutes
         /*
          * Initialize other stuff
@@ -170,7 +169,7 @@ public class HttpSessionWrapper implements HttpSession {
     }
 
     @Override
-    public Enumeration<String> getAttributeNames() {
+    public Enumeration<?> getAttributeNames() {
         return new IteratorEnumeration(attributes.keySet().iterator());
     }
 
@@ -301,11 +300,11 @@ public class HttpSessionWrapper implements HttpSession {
         }
     }
 
-    private static class IteratorEnumeration implements Enumeration<String> {
+    private static class IteratorEnumeration implements Enumeration<Object> {
 
-        private final Iterator<String> iter;
+        private final Iterator<?> iter;
 
-        public IteratorEnumeration(final Iterator<String> iter) {
+        public IteratorEnumeration(final Iterator<?> iter) {
             this.iter = iter;
         }
 
@@ -315,7 +314,7 @@ public class HttpSessionWrapper implements HttpSession {
         }
 
         @Override
-        public String nextElement() {
+        public Object nextElement() {
             return iter.next();
         }
     }
