@@ -160,7 +160,7 @@ public final class AJPv13ForwardRequest extends AbstractAJPv13Request {
         attributeMapping.put(0x0d, ATTR_STORED_METHOD);
         attributeMapping.put(REQUEST_TERMINATOR, "are_done");
 
-        final ConfigurationService configurationService = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
+        final ConfigurationService configurationService = Services.getService(ConfigurationService.class);
         if (configurationService == null) {
             forceHttps = false;
         } else {
@@ -402,9 +402,7 @@ public final class AJPv13ForwardRequest extends AbstractAJPv13Request {
                 // Look-up character '='
                 final int pos = paramsNVP.indexOf('=');
                 if (pos >= 0) {
-                    servletRequest.setParameter(paramsNVP.substring(0, pos), decodeQueryStringValue(
-                        servletRequest.getCharacterEncoding(),
-                        paramsNVP.substring(pos + 1)));
+                    servletRequest.setParameter(paramsNVP.substring(0, pos), decodeQueryStringValue(AJPv13Config.getServerProperty(Property.DefaultEncoding), paramsNVP.substring(pos + 1)));
                 } else {
                     servletRequest.setParameter(paramsNVP, STR_EMPTY);
                 }
