@@ -91,6 +91,7 @@ import com.openexchange.ajp13.exception.AJPv13MaxPackgeSizeException;
 import com.openexchange.ajp13.servlet.http.HttpErrorServlet;
 import com.openexchange.ajp13.servlet.http.HttpServletManager;
 import com.openexchange.ajp13.servlet.http.HttpSessionManagement;
+import com.openexchange.ajp13.util.IPTools;
 import com.openexchange.ajp13.watcher.AJPv13TaskMonitor;
 import com.openexchange.configuration.ServerConfig;
 import com.openexchange.configuration.ServerConfig.Property;
@@ -352,6 +353,11 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
      */
     private boolean restrictLongRunning;
 
+    /**
+     * Whether to consider X-Forwards header.
+     */
+    private final boolean isConsiderXForwards;
+
     /**
      * Direct buffer used for sending right away a pong message.
      */
@@ -423,6 +429,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
      */
     public AjpProcessor(final int packetSize, final AJPv13TaskMonitor listenerMonitor, final boolean forceHttps) {
         super();
+        isConsiderXForwards = AJPv13Config.isConsiderXForwards();
         restrictLongRunning = false;
         this.forceHttps = forceHttps;
         bodyBytes = MessageBytes.newInstance();
@@ -871,8 +878,6 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                      * Enter service stage...
                      */
                     lastWriteAccess = System.currentTimeMillis();
-                    stage = Stage.STAGE_SERVICE;
-                    listenerMonitor.incrementNumProcessing();
                     /*
                      * Form data?
                      */
@@ -913,16 +918,48 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                         error = true;
                     } else {
                        /*
-                        * Call Servlet's service() method
+                        * Check for special X-Forwards if enabled
                         */
-                        servlet.service(request, response);
-                        if (!started) {
-                            // Stopped in the meantime
-                            return;
+                        if (isConsiderXForwards) {
+                            // Scheme
+                            String protocol = request.getHeader(AJPv13Config.getProtocolHeader());
+                            if(!isValidProtocol(protocol)) {
+                                if (LOG.isDebugEnabled()) {
+                                    LOG.debug("Could not detect a valid protocol header value in " + protocol + ", falling back to default");
+                                }
+                                 protocol = request.getScheme();
+                            }
+                            // Remote address
+                            String forHeaderValue = request.getHeader(AJPv13Config.getForHeader());
+                            String remoteIP = IPTools.getRemoteIP(forHeaderValue, AJPv13Config.getKnownProxies());
+                            if (remoteIP.isEmpty()) {
+                                if (LOG.isDebugEnabled()) {
+                                    forHeaderValue = forHeaderValue == null ? "" : forHeaderValue;
+                                    LOG.debug("Could not detect a valid remote ip in " + AJPv13Config.getForHeader() + ": [" + forHeaderValue + "], falling back to default");
+                                }
+                                remoteIP = request.getRemoteAddr();
+                            }
+                            // Apply values
+                            request.setScheme(protocol);
+                            request.setRemoteAddr(remoteIP);
+                        }
+                        /*
+                         * Call Servlet's service() method
+                         */
+                        listenerMonitor.incrementNumProcessing();
+                        try {
+                            stage = Stage.STAGE_SERVICE;
+                            servlet.service(request, response);
+                            if (!started) {
+                                // Stopped in the meantime
+                                return;
+                            }
+                            response.flushBuffer();
+                            listenerMonitor.addProcessingTime(System.currentTimeMillis() - request.getStartTime());
+                            listenerMonitor.incrementNumRequests();
+                        } finally {
+                            listenerMonitor.decrementNumProcessing();
                         }
-                        response.flushBuffer();
-                        listenerMonitor.addProcessingTime(System.currentTimeMillis() - request.getStartTime());
-                        listenerMonitor.incrementNumRequests();
                     }
                 } catch (final UploadServletException e) {
                     /*
@@ -957,7 +994,6 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     error = true;
                 } finally {
                     stage = Stage.STAGE_SERVICE_ENDED;
-                    listenerMonitor.decrementNumProcessing();
                     if (longRunningAccepted) {
                         AjpLongRunningRegistry.getInstance().deregisterLongRunning(request);
                     }
@@ -1095,7 +1131,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                     if (finished || (max <= 0) || (!isProcessing() || ((System.currentTimeMillis() - getLastWriteAccess()) <= max))) {
                         return;
                     }
-                    boolean doReadMessage = true;
+                    final boolean doReadMessage = true;
                     if (response.isCommitted()) {
                         /*
                          * Write empty SEND-BODY-CHUNK package
@@ -1233,6 +1269,14 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         return servlet;
     }
 
+    private final static String HTTP_SCHEME = "http";
+
+    private final static String HTTPS_SCHEME = "https";
+
+    private boolean isValidProtocol(final String protocolHeaderValue) {
+        return HTTP_SCHEME.equals(protocolHeaderValue) || HTTPS_SCHEME.equals(protocolHeaderValue);
+    }
+
     private static final String JSESSIONID_URI = AJPv13RequestHandler.JSESSIONID_URI;
 
     private static final String EAS_URI = "/Microsoft-Server-ActiveSync";
@@ -1650,17 +1694,14 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
      */
     private void parseQueryString(final String queryStr) {
         final String[] paramsNVPs = PATTERN_SPLIT.split(queryStr, 0);
-        String charEnc = request.getCharacterEncoding();
-        if (null == charEnc) {
-            charEnc = AJPv13Config.getServerProperty(Property.DefaultEncoding);
-        }
+        final String defaultCharEnc = AJPv13Config.getServerProperty(Property.DefaultEncoding);
         for (String paramsNVP : paramsNVPs) {
             paramsNVP = paramsNVP.trim();
             if (paramsNVP.length() > 0) {
                 // Look-up character '='
                 final int pos = paramsNVP.indexOf('=');
                 if (pos >= 0) {
-                    request.setParameter(paramsNVP.substring(0, pos), decodeUrl(paramsNVP.substring(pos + 1), charEnc));
+                    request.setParameter(paramsNVP.substring(0, pos), decodeUrl(paramsNVP.substring(pos + 1), defaultCharEnc));
                 } else {
                     request.setParameter(paramsNVP, "");
                 }
@@ -1674,10 +1715,12 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
      * @param requestURI The request URI
      */
     private void setServletInstance(final String requestURI) {
+        final String defaultCharEnc = AJPv13Config.getServerProperty(Property.DefaultEncoding);
+        final String decodedRequestURI = AJPv13Utility.decodeUrl(requestURI, defaultCharEnc);
         /*
          * Remove leading slash character
          */
-        final String path = requestURI.length() > 1 ? removeFromPath(requestURI, '/') : requestURI;
+        final String path = decodedRequestURI.length() > 1 ? removeFromPath(decodedRequestURI, '/') : decodedRequestURI;
         /*
          * Lookup path in available servlet paths
          */
@@ -1686,7 +1729,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
         }
         HttpServlet servlet = HttpServletManager.getServlet(path, servletId);
         if (servlet == null) {
-            servlet = new HttpErrorServlet("No servlet bound to path/alias: " + AJPv13Utility.urlEncode(requestURI));
+            servlet = new HttpErrorServlet("No servlet bound to path/alias: " + AJPv13Utility.urlEncode(decodedRequestURI, defaultCharEnc));
         }
         this.servlet = servlet;
         // servletId = pathStorage.length() > 0 ? pathStorage.toString() : null;
@@ -1708,7 +1751,7 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                 /*
                  * Set complete request URI as path info
                  */
-                request.setPathInfo(requestURI);
+                request.setPathInfo(decodedRequestURI);
             } else {
                 /*
                  * The path starts with a "/" character and includes either the servlet name or a path to the servlet, but does not include
@@ -1719,8 +1762,8 @@ public final class AjpProcessor implements com.openexchange.ajp13.watcher.Task {
                  * Set path info: The extra path information follows the servlet path but precedes the query string and will start with a
                  * "/" character.
                  */
-                if ((requestURI.length() > servletPathLen) /* && requestURI.startsWith(servletPath) */) {
-                    request.setPathInfo(requestURI.substring(servletPathLen));
+                if ((decodedRequestURI.length() > servletPathLen) /* && requestURI.startsWith(servletPath) */) {
+                    request.setPathInfo(decodedRequestURI.substring(servletPathLen));
                 } else {
                     request.setPathInfo(null);
                 }
